<?php
namespace Drupal\ot\Form;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\Core\Url;
use \Drupal\views\Views;
use Drupal\ot\Controller\OverrideMain;

class Overridetitle extends FormBase
{

  public function __construct()
  {
    $this->ot_main = new OverrideMain();
  }

  public function getFormId()
  {
    return 'override_title';
  }

  public function buildForm(array $form, FormStateInterface $form_state)
  {
    $form['#attached']['library'][] = 'ot/ot.lib';

    $form['ot_action'] = array(
      '#type'=> 'select',
      '#title'=> t('Action'),
      '#options'=> $this->ot_main->getOtAction()
    );

    $form['submit'] = array(
      '#type'=> 'submit',
      '#value'=> t('Apply to selected items'),
      '#prefix'=> '<div class="form-item">',
      '#suffix'=> '</div>',
      '#attributes'=> array('class'=> ['ot-del'], 'onclick'=> 'return otDelete();'),
    );

    $param = \Drupal::request()->query->all();
    if(!empty($param)){
      $order = $param['order'];
      $sort = $param['sort'];
    }else{
      $order = 'created';
      $sort = 'desc';
    }

    $form['markup']= array(
      '#type'=> 'markup',
      '#title'=> t('From'),
      '#markup'=> '<a href="#" id="ot-show-hide">#Show items</a>',
    );

    $data = $this->ot_main->OtGetSortedData($order, $sort);

    $form['ot_modify_multiple'] = array(
      '#type' => 'tableselect',
      '#header' => $data[0],
      '#options' => !empty($data[1]) ? $data[1] : [],
      '#empty' => t('No Override Title found yet.'),
    );

    $form['pager'] = array(
      '#type' => 'pager'
    );

    return $form;
  }

  public function validateForm(array &$form, FormStateInterface $form_state)
  {
    $action_ot = $form_state->getValue('ot_action');
    $modify_ot = $form_state->getValue('ot_modify_multiple');
    $modify_ot_arr = array_diff($modify_ot, [0]);
    $modify_ot_check = implode('', $modify_ot_arr);
    $modify_ot_arr_val = array_values($modify_ot_arr);

    if(empty($modify_ot_arr)){
      $form_state->setErrorByName('ot_modify_multiple', t('No Override Title content selected.'));
    }
    else if(!is_numeric($modify_ot_check)){
      $form_state->setErrorByName('ot_modify_multiple', t('An illegal choice has been detected. Please contact the site administrator.'));
    }
    else{
      $empty_check = array_diff($modify_ot_arr_val, $this->ot_main->getOtByIdMultiple($modify_ot_arr_val));
      if(!empty($empty_check)){
        $form_state->setErrorByName('ot_modify_multiple', t('An illegal choice has been detected. Please contact the site administrator.'));
      }
    }

    if(!in_array($action_ot, $this->ot_main->checkOtAction())){
      $form_state->setErrorByName('ot_action', t('An illegal choice has been detected. Please contact the site administrator.'));
    }

  }

  public function submitForm(array &$form, FormStateInterface $form_state)
  {
    $action_ot = $form_state->getValue('ot_action');
    $modify_ot = $form_state->getValue('ot_modify_multiple');
    $modify_ot_arr = array_diff($modify_ot, [0]);
    $modify_ot_arr_val = array_values($modify_ot_arr);
    $s = (count($modify_ot_arr_val) >1) ? ' items.' : ' item.';
    if(in_array($action_ot, ['ot_delete'])){
      $success = $this->ot_main->OtDeleteMultiple($modify_ot_arr_val);
      if($success){
        drupal_set_message(t('Deleted '.count($modify_ot_arr_val).' Override Title content'.$s.'.'), 'status');
      }
    }
    else if(in_array($action_ot, ['ot_active', 'ot_deactive'])){
      $status = ['ot_active'=> 1, 'ot_deactive'=> 0];
      $status_string = ['ot_active'=> 'Published', 'ot_deactive'=> 'UnPublished'];
      $success = $this->ot_main->changeOtStatus($modify_ot_arr_val, $status[$action_ot]);
      if($success){
        drupal_set_message(t($status_string[$action_ot].' Override Title content was applied to '.count($modify_ot_arr_val).$s), 'status');
      }
    }
    else if(in_array($action_ot, $this->ot_main->checkOtLocation())){
      $success = $this->ot_main->changeOtLocation($modify_ot_arr_val, $action_ot);
      if($success){
        drupal_set_message(t('Location change of Override Title content was applied to '.count($modify_ot_arr_val).$s), 'status');
      }
    }

    if(!$success){
      drupal_set_message(t('Your process could not completed. Please try again.'), 'error');
    }

    $url = Url::fromRoute('override.ot');
    $form_state->setRedirectUrl($url);
  }

}
